@@ -886,7 +886,10 @@ calmaElementText()
 	int flags, i;
 	Label *lab;
 
-	if (cifnum >= 0 && (cifCurReadStyle->crs_labelSticky[cifnum] != LABEL_TYPE_NONE))
+        if (type == TT_SPACE)
+	    /* Assigning GDS layer to space prevents making the label sticky */
+	    flags = 0;
+	else if (cifnum >= 0 && (cifCurReadStyle->crs_labelSticky[cifnum] != LABEL_TYPE_NONE))
 	    flags = LABEL_STICKY;
 	else if (cifCurReadStyle->crs_flags & CRF_NO_RECONNECT_LABELS)
 	    flags = LABEL_STICKY;
