@@ -20,10 +20,11 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBconnect.c,v 1.5 2010/06/24 12:37:15 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBconnect.c,v 1.6 2010/09/15 18:15:40 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>		// for memcpy()
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -131,6 +132,43 @@ DBTransformDiagonal(oldtype, trans)
     return dinfo;
 }
 
+/*
+ *-----------------------------------------------------------------
+ * DBInvTransformDiagonal --
+ *
+ *	This is equivalent to the routine above, but inverted, so
+ *	that the result is correct for the orientation of the
+ *	triangle in the coordinate system of the child cell,
+ *	rather than the parent cell (which comes down to merely
+ *	swapping transform positions b and d, since translation
+ *	isn't considered).
+ *-----------------------------------------------------------------
+ */
+
+TileType
+DBInvTransformDiagonal(oldtype, trans)
+    TileType oldtype;
+    Transform *trans;
+{
+    TileType dinfo;
+    int o1, o2, o3;
+    int dir, side;
+
+    o1 = ((trans->t_e > 0) || (trans->t_b > 0)) ? 1 : 0;
+    o2 = ((trans->t_a > 0) || (trans->t_d > 0)) ? 1 : 0;
+    o3 = (trans->t_a != 0) ? 1 : 0;
+
+    dir = (oldtype & TT_DIRECTION) ? 1 : 0;
+    side = ((oldtype & TT_SIDE) ? 1 : 0) ^ o2 ^ (dir | o3);
+    dir ^= o1 ^ o2;
+
+    dinfo = TT_DIAGONAL;
+    if (side) dinfo |= TT_SIDE;
+    if (dir) dinfo |= TT_DIRECTION;
+
+    return dinfo;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -222,6 +260,9 @@ DBSrConnect(def, startArea, mask, connect, bounds, func, clientData)
 	    dbSrConnectStartFunc, (ClientData) &startTile) != 0) break;
     }
     if (startTile == NULL) return 0;
+    /* The following lets us call DBSrConnect recursively */
+    else if (startTile->ti_client == (ClientData)1) return 0;
+   
 
     /* Pass 1.  During this pass the client function gets called. */
 
@@ -255,6 +296,77 @@ dbSrConnectStartFunc(tile, pTile)
     return 1;
 }
 
+/* Function similar to DBSrConnect but which does the first pass only	*/
+/* and leaves the marked tiles intact.  Tiles must be cleared by the	*/
+/* caller.								*/
+
+int
+DBSrConnectOnePass(def, startArea, mask, connect, bounds, func, clientData)
+    CellDef *def;		/* Cell definition in which to carry out
+				 * the connectivity search.  Only paint
+				 * in this definition is considered.
+				 */
+    Rect *startArea;		/* Area to search for an initial tile.  Only
+				 * tiles OVERLAPPING the area are considered.
+				 * This area should have positive x and y
+				 * dimensions.
+				 */
+    TileTypeBitMask *mask;	/* Only tiles of one of these types are used
+				 * as initial tiles.
+				 */
+    TileTypeBitMask *connect;	/* Pointer to a table indicating what tile
+				 * types connect to what other tile types.
+				 * Each entry gives a mask of types that
+				 * connect to tiles of a given type.
+				 */
+    Rect *bounds;		/* Area, in coords of scx->scx_use->cu_def,
+				 * that limits the search:  only tiles
+				 * overalapping this area will be returned.
+				 * Use TiPlaneRect to search everywhere.
+				 */
+    int (*func)();		/* Function to apply at each connected tile. */
+    ClientData clientData;	/* Client data for above function. */
+
+{
+    struct conSrArg csa;
+    int startPlane, result;
+    Tile *startTile;			/* Starting tile for search. */
+    extern int dbSrConnectFunc();	/* Forward declaration. */
+    extern int dbSrConnectStartFunc();
+
+    result = 0;
+    csa.csa_def = def;
+    csa.csa_bounds = *bounds;
+
+    /* Find a starting tile (if there are many tiles underneath the
+     * starting area, pick any one).  The search function just saves
+     * the tile address and returns.
+     */
+
+    startTile = NULL;
+    for (startPlane = PL_TECHDEPBASE; startPlane < DBNumPlanes; startPlane++)
+    {
+	if (DBSrPaintArea((Tile *) NULL,
+	    def->cd_planes[startPlane], startArea, mask,
+	    dbSrConnectStartFunc, (ClientData) &startTile) != 0) break;
+    }
+    if (startTile == NULL) return 0;
+    /* The following lets us call DBSrConnect recursively */
+    else if (startTile->ti_client == (ClientData)1) return 0;
+   
+
+    /* Pass 1.  During this pass the client function gets called. */
+
+    csa.csa_clientFunc = func;
+    csa.csa_clientData = clientData;
+    csa.csa_clear = FALSE;
+    csa.csa_connect = connect;
+    csa.csa_plane = startPlane;
+    if (dbSrConnectFunc(startTile, &csa) != 0) result = 1;
+
+    return result;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -361,6 +473,8 @@ dbSrConnectFunc(tile, csa)
 		if (t2->ti_client == (ClientData) CLIENTDEFAULT) continue;
 	    }
 	    else if (t2->ti_client != (ClientData) CLIENTDEFAULT) continue;
+	    if (IsSplit(t2))
+		TiSetBody(t2, (ClientData)(t2->ti_body | TT_SIDE)); /* bit set */
 	    if (dbSrConnectFunc(t2, csa) != 0) return 1;
 	}
     }
@@ -386,6 +500,13 @@ bottomside:
 		if (t2->ti_client == (ClientData) CLIENTDEFAULT) continue;
 	    }
 	    else if (t2->ti_client != (ClientData) CLIENTDEFAULT) continue;
+	    if (IsSplit(t2))
+	    {
+		if (SplitDirection(t2))
+		    TiSetBody(t2, (ClientData)(t2->ti_body | TT_SIDE)); /* bit set */
+		else
+		    TiSetBody(t2, (ClientData)(t2->ti_body & ~TT_SIDE)); /* bit clear */
+	    }
 	    if (dbSrConnectFunc(t2, csa) != 0) return 1;
 	}
     }
@@ -410,6 +531,8 @@ rightside:
 		if (t2->ti_client == (ClientData) CLIENTDEFAULT) goto nextRight;
 	    }
 	    else if (t2->ti_client != (ClientData) CLIENTDEFAULT) goto nextRight;
+	    if (IsSplit(t2))
+		TiSetBody(t2, (ClientData)(t2->ti_body & ~TT_SIDE)); /* bit clear */
 	    if (dbSrConnectFunc(t2, csa) != 0) return 1;
 	}
 	nextRight: if (BOTTOM(t2) <= tileArea.r_ybot) break;
@@ -435,6 +558,13 @@ topside:
 		if (t2->ti_client == (ClientData) CLIENTDEFAULT) goto nextTop;
 	    }
 	    else if (t2->ti_client != (ClientData) CLIENTDEFAULT) goto nextTop;
+	    if (IsSplit(t2))
+	    {
+		if (SplitDirection(t2))
+		    TiSetBody(t2, (ClientData)(t2->ti_body & ~TT_SIDE)); /* bit clear */
+		else
+		    TiSetBody(t2, (ClientData)(t2->ti_body | TT_SIDE)); /* bit set */
+	    }
 	    if (dbSrConnectFunc(t2, csa) != 0) return 1;
 	}
 	nextTop: if (LEFT(t2) <= tileArea.r_xbot) break;
@@ -458,6 +588,7 @@ donesides:
 	GEO_EXPAND(&newArea, 1, &newArea);
 	for (i = PL_TECHDEPBASE; i < DBNumPlanes; i++)
 	{
+	    if (!PlaneMaskHasPlane(planes, i)) continue;
 	    newcsa.csa_plane = i;
 	    if (IsSplit(tile))
 	    {
@@ -543,9 +674,15 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
     GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &offset);
     rotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
 
-    DBEraseLabelsByContent(def, &r, -1, lab->lab_text);
-    DBPutFontLabel(def, &r, lab->lab_font, lab->lab_size, rotate, &offset,
+    /* Only add labels if they are on the search top level  */
+    /* (NOTE:  Could add hierachical labels using tpath)    */
+
+    if (scx->scx_use == csa2->csa2_topscx->scx_use)
+    {
+	DBEraseLabelsByContent(def, &r, -1, lab->lab_text);
+	DBPutFontLabel(def, &r, lab->lab_font, lab->lab_size, rotate, &offset,
 		pos, lab->lab_text, lab->lab_type, lab->lab_flags);
+    }
 
     if (lab->lab_flags & PORT_DIR_MASK)
     {
@@ -564,8 +701,16 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
 		    SearchContext scx2 = *csa2->csa2_topscx;
 		    TileTypeBitMask mask;
 
+		    // Do NOT go searching on labels connected to space!
+		    if (slab->lab_type == TT_SPACE) continue;
 		    TTMaskSetOnlyType(&mask, slab->lab_type);
 		    GeoTransRect(&scx->scx_trans, &slab->lab_rect, &scx2.scx_area);
+
+		    // Expand search area by 1 to capture edge and point labels.
+		    scx2.scx_area.r_xbot--;
+		    scx2.scx_area.r_xtop++;
+		    scx2.scx_area.r_ybot--;
+		    scx2.scx_area.r_ytop++;
 		    DBTreeSrTiles(&scx2, &mask, csa2->csa2_xMask,
 				dbcConnectFunc, (ClientData) csa2);
 		}
@@ -615,6 +760,7 @@ dbcConnectFunc(tile, cx)
     TileType loctype = TiGetTypeExact(tile);
     TileType dinfo = 0;
     int pNum = cx->tc_plane;
+    unsigned char searchtype;
     CellDef *def;
 
     TiToRect(tile, &tileArea);
@@ -663,32 +809,37 @@ dbcConnectFunc(tile, cx)
 
     if (DBIsContact(loctype))
     {
-	TileType ctype;
-	TileTypeBitMask *cMask, *rMask = DBResidueMask(loctype);
+//	TileType ctype;
+//	TileTypeBitMask *cMask, *rMask = DBResidueMask(loctype);
 
-	TTMaskSetOnlyType(&notConnectMask, loctype);
+//	TTMaskSetOnlyType(&notConnectMask, loctype);
 
 	/* Different contact types may share residues (6/18/04) */
 	/* Use TTMaskIntersect(), not TTMaskEqual()---types	*/
 	/* which otherwise stack may be in separate cells	*/
 	/* (12/1/05)						*/
 
-	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
-	{
-	    cMask = DBResidueMask(ctype);
-	    if (TTMaskIntersect(rMask, cMask))
-		TTMaskSetType(&notConnectMask, ctype);
-	}
+//	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
+//	{
+//	    if (DBIsContact(ctype))
+//	    {
+//		cMask = DBResidueMask(ctype);
+//		if (TTMaskIntersect(rMask, cMask))
+//		    TTMaskSetType(&notConnectMask, ctype);
+//	    }
+//	}
 
 	/* The mask of contact types must include all stacked contacts */
-	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
-	{
-	    cMask = DBResidueMask(ctype);
-	    if (TTMaskHasType(cMask, loctype))
-		TTMaskSetType(&notConnectMask, ctype);
-	}
+//	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
+//	{
+//	    cMask = DBResidueMask(ctype);
+//	    if (TTMaskHasType(cMask, loctype))
+//		TTMaskSetType(&notConnectMask, ctype);
+//	}
+//	TTMaskCom(&notConnectMask);
 
-	TTMaskCom(&notConnectMask);
+	TTMaskZero(&notConnectMask);
+	TTMaskSetMask(&notConnectMask, &DBNotConnectTbl[loctype]);
     }
     else
     {
@@ -713,15 +864,41 @@ dbcConnectFunc(tile, cx)
 		&newarea, DBStdPaintTbl(loctype, pNum),
 		(PaintUndoInfo *) NULL);
 
-    /* Copy information from original context into new search context */
-    scx2 = *csa2->csa2_topscx;
-    scx2.scx_area = newarea;
-
     /* Check the source def for any labels belonging to this	*/
     /* tile area and plane, and add them to the destination.	*/
 
+    searchtype = TF_LABEL_ATTACH;
+    if (IsSplit(tile))
+    {
+	/* If the tile is split, then labels attached to the	*/
+	/* opposite point of the triangle are NOT connected.	*/
+
+	if (SplitSide(tile))
+	{
+	    if (SplitDirection(tile))
+		searchtype |= TF_LABEL_ATTACH_NOT_SW;
+	    else
+		searchtype |= TF_LABEL_ATTACH_NOT_NW;
+	}
+	else
+	{
+	    if (SplitDirection(tile))
+		searchtype |= TF_LABEL_ATTACH_NOT_NE;
+	    else
+		searchtype |= TF_LABEL_ATTACH_NOT_SE;
+	}
+    }
+
+    /* Note that the search must be done from the top since zero-size	*/
+    /* port labels can be on any part of the hierarchy with no paint	*/
+    /* underneath in its own cell to trigger the callback function.	*/
+
+    /* Copy information from top search context into new search context */
+    scx2 = *csa2->csa2_topscx;
+    scx2.scx_area = newarea;
+
     DBTreeSrLabels(&scx2, connectMask, csa2->csa2_xMask, NULL,
-			TF_LABEL_ATTACH, dbcConnectLabelFunc,
+			searchtype, dbcConnectLabelFunc,
 			(ClientData) csa2);
 
     /* Since the whole area of this tile hasn't been recorded,
@@ -768,12 +945,14 @@ dbcConnectFunc(tile, cx)
 	csa2->csa2_size *= 2;
 
 	newlist = (conSrArea *)mallocMagic(csa2->csa2_size * sizeof(conSrArea));
-	for (i = 0; i < lastsize; i++)
-	{
-	    newlist[i].area = csa2->csa2_list[i].area;
-	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
-	    newlist[i].dinfo = csa2->csa2_list[i].dinfo;
-	}
+	memcpy((void *)newlist, (void *)csa2->csa2_list,
+			(size_t)lastsize * sizeof(conSrArea));
+	// for (i = 0; i < lastsize; i++)
+	// {
+	//     newlist[i].area = csa2->csa2_list[i].area;
+	//     newlist[i].connectMask = csa2->csa2_list[i].connectMask;
+	//     newlist[i].dinfo = csa2->csa2_list[i].dinfo;
+	// }
 	freeMagic((char *)csa2->csa2_list);
 	csa2->csa2_list = newlist;
     }
