@@ -155,11 +155,11 @@ typedef struct def
     int		 def_flags;	/* Flags -- see below */
     HashTable	 def_nodes;	/* Map names into EFNodeNames */
     HashTable	 def_dists;	/* Map pairs of names into Distances */
+    HashTable	 def_uses;	/* Hash children of this def by name */
     EFNode	 def_firstn;	/* Head of circular list of nodes */
 
 	/* The following are all NULL-terminated lists */
 
-    struct use	*def_uses;	/* Children of this def */
     Connection	*def_conns;	/* Hierarchical connections/adjustments */
     Connection	*def_caps;	/* Two-terminal capacitors */
     Connection	*def_resistors;	/* Two-terminal resistors */
@@ -178,6 +178,7 @@ typedef struct def
 #define DEF_PROCESSED	0x04	/* This def processed in hierarchical output */
 #define DEF_NODEVICES	0x08	/* This def contains no devices */
 #define DEF_SUBSNODES	0x10	/* This def contains implicit substrate nodes */
+#define DEF_ABSTRACT	0x20	/* This def is an abstract view (e.g., LEF) */
 
 /*
  * Every Def has a NULL-terminated list of uses that correspond
@@ -198,8 +199,6 @@ typedef struct use
 {
     char	*use_id;	/* Use identifier (appears in hier paths) */
     Def		*use_def;	/* Sub def being used */
-    // EFNodeName *use_ports;	/* Port connections, for hierarchical output */
-    struct use	*use_next;	/* Next use in list (NULL-terminated) */
     Transform	 use_trans;	/* Transform up to parent coords (for fets) */
     ArrayInfo	 use_array;	/* Arraying information */
 } Use;
