@@ -29,6 +29,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include <stdio.h>
 #include <string.h>
 
+#include "tcltk/tclmagic.h"
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
@@ -81,6 +82,7 @@ ExtTree *extSubList = (ExtTree *) NULL;
 /* Forward declarations of filter functions */
 char *extSubtreeTileToNode();
 int extSubtreeFunc();
+int extSubstrateFunc();
 int extConnFindFunc();
 int extSubtreeHardUseFunc();
 int extHardProc();
@@ -141,6 +143,8 @@ extSubtree(parentUse, reg, f)
     Rect r, rlab, rbloat, *b;
     Label *lab;
     bool result;
+    int cuts, totcuts;
+    float pdone, plast;
 
     if ((ExtOptions & (EXT_DOCOUPLING|EXT_DOADJUST))
 		   != (EXT_DOCOUPLING|EXT_DOADJUST))
@@ -171,7 +175,18 @@ extSubtree(parentUse, reg, f)
      * halo has been set above to reflect the maximum distance for
      * sidewall coupling capacitance).
      */
+
     b = &def->cd_bbox;
+
+    /* Monitor progress, for large designs */
+    totcuts = (b->r_ytop - b->r_ybot + ExtCurStyle->exts_stepSize - 1)
+		    / ExtCurStyle->exts_stepSize;
+    totcuts *= ((b->r_xtop - b->r_xbot + ExtCurStyle->exts_stepSize - 1)
+		    / ExtCurStyle->exts_stepSize);
+    cuts = 0;
+    pdone = 0.0;
+    plast = 0.0;
+
     for (r.r_ybot = b->r_ybot; r.r_ybot < b->r_ytop; r.r_ybot = r.r_ytop)
     {
 	r.r_ytop = r.r_ybot + ExtCurStyle->exts_stepSize;
@@ -212,6 +227,30 @@ extSubtree(parentUse, reg, f)
 		extSubtreeClippedArea += RECTAREA(&ha.ha_clipArea);
 		extSubtreeInteraction(&ha);
 	    }
+	    else
+	    {
+		/* Make sure substrate connections have been handled	*/
+		/* even if there were no other interactions found.	*/
+		SearchContext scx;
+
+		scx.scx_trans = GeoIdentityTransform;
+		scx.scx_area = ha.ha_interArea;
+		scx.scx_use = ha.ha_parentUse;
+		DBCellSrArea(&scx, extSubstrateFunc, (ClientData)&ha);
+	    }
+
+	    cuts++;
+	    pdone = 100.0 * ((float)cuts / (float)totcuts);
+	    if ((((pdone - plast) > 5.0) || (cuts == totcuts)) && (cuts > 1)) {
+	        TxPrintf("Completed %d%%\n", (int)(pdone + 0.5));
+	        plast = pdone;
+		TxFlushOut();
+
+#ifdef MAGIC_WRAPPER
+		/* We need to let Tk paint the console display */
+		while (Tcl_DoOneEvent(TCL_DONT_WAIT) != 0);
+#endif
+	    }
 	}
     }
 #else	/* exactinteractions */
@@ -790,6 +829,63 @@ extSubtreeFunc(scx, ha)
     return (2);
 }
 
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extSubstrateFunc
+ *
+ * This contains just the part of extSubtreeFunc() dealing with the
+ * substrate, so that substrate extraction can occur in cells not
+ * otherwise having extraction interactions, without incurring the
+ * overhead of all the other items handled by extHierSubtreeFunc().
+ *
+ * Results:
+ *	Always returns 2, to avoid further elements in arrays.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+extSubstrateFunc(scx, ha)
+    SearchContext *scx;
+    HierExtractArg *ha;
+{
+    CellUse *use = scx->scx_use;
+    int x, y;
+
+    /* Record information for finding node names the hard way */
+    ha->ha_subUse = use;
+    ha->ha_subArea = use->cu_bbox;
+    GEOCLIP(&ha->ha_subArea, &ha->ha_interArea);
+
+    /* Process substrate connection.  All substrates should be	*/
+    /* connected together in the cell def, so in the case of an	*/
+    /* array, just make sure that the first array entry is	*/
+    /* connected.						*/
+    
+    if (use->cu_xhi == use->cu_xlo && use->cu_yhi == use->cu_ylo)
+	extHierSubstrate(ha, use, -1, -1);
+    else if (use->cu_xhi == use->cu_xlo && use->cu_yhi > use->cu_ylo)
+    {	
+	for (y = use->cu_ylo; y <= use->cu_yhi; y++)
+	    extHierSubstrate(ha, use, -1, y);
+    }
+    else if (use->cu_xhi > use->cu_xlo && use->cu_yhi == use->cu_ylo)
+    {
+	for (x = use->cu_xlo; x <= use->cu_xhi; x++)
+	extHierSubstrate(ha, use, x, -1);
+    }
+    else
+    {
+	for (x = use->cu_xlo; x <= use->cu_xhi; x++)
+	    for (y = use->cu_ylo; y <= use->cu_yhi; y++)
+		extHierSubstrate(ha, use, x, y);
+    }
+    return (2);
+}
+
+
 /*
  * ----------------------------------------------------------------------------
  *
