@@ -164,8 +164,6 @@ lefWriteHeader(def, f, lefTech)
 {
     TileType type;
 
-    UndoDisable();
-
     TxPrintf("Diagnostic:  Write LEF header for cell %s\n", def->cd_name);
 
     /* NOTE:  This routine corresponds to Envisia LEF/DEF Language	*/
@@ -174,7 +172,6 @@ lefWriteHeader(def, f, lefTech)
     fprintf(f, "VERSION 5.3 ;\n");
     fprintf(f, "   NAMESCASESENSITIVE ON ;\n");
     fprintf(f, "   NOWIREEXTENSIONATPIN ON ;\n");
-    fprintf(f, "   NOWIREEXTENSIONATPIN ON ;\n");
     fprintf(f, "   DIVIDERCHAR \"/\" ;\n");
     fprintf(f, "   BUSBITCHARS \"[]\" ;\n");
 
@@ -189,6 +186,8 @@ lefWriteHeader(def, f, lefTech)
 
     if (!lefTech) return;
 
+    UndoDisable();
+
     /* Layers (minimal information) */
 
     if (LefInfo.ht_table != (HashEntry **)NULL)
@@ -421,7 +420,7 @@ lefYankGeometry(tile, cdata)
 
     TiToRect(tile, &area);
 
-    while (ttype < DBNumTypes)
+    while (ttype < DBNumUserLayers)
     {
 	lefMagicToLefLayer = lefdata->lefMagicMap;
 	if (lefMagicToLefLayer[ttype].lefInfo != NULL)
